-- Top 20 tables in database, by size:

SELECT oid::REGCLASS::TEXT AS table_name,
       pg_size_pretty(pg_total_relation_size(oid)) AS total_size
  FROM pg_class
 WHERE relkind = 'r'
   AND relpages > 0
 ORDER BY pg_total_relation_size(oid) DESC
 LIMIT 20;

-- Top 20 indexes in database, by size:

SELECT indexrelid::REGCLASS::TEXT AS index_name,
       indrelid::REGCLASS::TEXT AS table_name,
       pg_size_pretty(pg_relation_size(indexrelid)) AS total_size
  FROM pg_index
 ORDER BY pg_relation_size(indexrelid) DESC
 LIMIT 20;

-- Top 20 tables in database, by write activity:

SELECT relid::REGCLASS AS table_name,
	   n_tup_ins AS inserts,
	   n_tup_upd + n_tup_hot_upd AS updates,
	   n_tup_del AS deletes
  FROM pg_stat_user_tables
 ORDER BY (n_tup_ins + n_tup_upd + n_tup_hot_upd + n_tup_del) DESC
 LIMIT 20;

-- Top 20 tables in database, by read activity:

SELECT relid::REGCLASS AS table_name,
       coalesce(seq_scan, 0) AS sequence_scans,
       coalesce(idx_scan, 0) AS index_scans,
       coalesce(seq_tup_read, 0) AS table_matches,
       coalesce(idx_tup_fetch, 0) AS index_matches
  FROM pg_stat_user_tables
 ORDER BY (coalesce(seq_scan, 0) + coalesce(idx_scan, 0)) DESC,
      (coalesce(seq_tup_read, 0) + coalesce(idx_tup_fetch, 0)) DESC
 LIMIT 20;

-- Top 20 indexes in database, by read activity:

SELECT indexrelid::REGCLASS AS index_name,
       relid::REGCLASS AS table_name,
       coalesce(idx_scan, 0) AS index_scans,
       coalesce(idx_tup_read, 0) AS rows_read,
       coalesce(idx_tup_fetch, 0) AS rows_fetched
  FROM pg_stat_user_indexes
 ORDER BY (coalesce(idx_scan, 0) + coalesce(idx_tup_read, 0)) DESC
 LIMIT 20;

-- Reset all activity stats.

SELECT pg_stat_reset();

-- Obtain full path to most active tables.

DROP TABLE IF EXISTS active_snap;

CREATE TABLE active_snap AS
(SELECT t.relid AS objrelid,
        s.setting || '/' || pg_relation_filepath(t.relid) AS file_path
   FROM pg_stat_user_tables t, pg_settings s
  WHERE s.name = 'data_directory'
  ORDER BY coalesce(idx_scan, 0) DESC
 LIMIT 20)
UNION
(SELECT t.indexrelid AS objrelid,
        s.setting || '/' || pg_relation_filepath(t.indexrelid) AS file_path
   FROM pg_stat_user_indexes t, pg_settings s
  WHERE s.name = 'data_directory'
  ORDER BY coalesce(idx_scan, 0) DESC
  LIMIT 20);

COPY active_snap TO '/tmp/frequent_tables.txt';

-- Fill caches with pgFincore.
  
DO $$
DECLARE
  obj_oid oid;
BEGIN
  FOR obj_oid IN SELECT objrelid FROM active_snap
  LOOP
    PERFORM pgfadvise_willneed(obj_oid::regclass);
  END LOOP;
END;
$$ LANGUAGE plpgsql;

-- Disable database connections.

UPDATE pg_database
   SET datallowconn = FALSE
 WHERE datname != 'template1';

-- Enable database connections.

UPDATE pg_database
   SET datallowconn = TRUE;

-- Find client disconnection candidates.

SELECT pid, client_port, state,
       now() - query_start AS duration, query
  FROM pg_stat_activity
 WHERE now() - query_start > INTERVAL '2 seconds'
   AND state != 'idle'
 ORDER BY duration DESC;

-- The same query for 9.1 systems and below:

SELECT procpid AS pid, client_port,
       now() - query_start AS duration, current_query
  FROM pg_stat_activity
 WHERE now() - query_start > INTERVAL '2 seconds'
   AND current_query != '<IDLE>'
 ORDER BY duration DESC;

-- Automatically kill several queries.

SELECT pg_terminate_backend(pid)
  FROM pg_stat_activity
 WHERE now() - query_start > INTERVAL '2 minutes'
   AND state = 'idle in transaction';

-- Concurrent index creation:

CREATE INDEX CONCURRENTLY idx_account_bid
    ON pgbench_accounts (bid);

-- Test insert:

INSERT INTO pgbench_accounts
VALUES (50000000, 100, 15000, 'testing');

-- Create a replication user:

CREATE USER rep_user WITH PASSWORD 'rep_test' REPLICATION;

-- Check replication status on main server:

SELECT sent_location, replay_location
  FROM pg_stat_replication
 WHERE usename = 'rep_user';


